package {
	
	import flash.display.Sprite;
	import com.fileitup.fisixengine.core.FisixEngine;
	import com.fileitup.fisixengine.collisions.ReactionModes;
	import flash.display.GradientType;
	import flash.geom.Matrix;
	import com.fileitup.fisixengine.primitives.Surface;
	import com.fileitup.fisixengine.core.Vector;
	import fisixObjects.Truck;
	import flash.display.Bitmap;
	import com.fileitup.fisixengine.graphics.WheelAttacher;
	import com.fileitup.fisixengine.graphics.ParticleAttacher;
	import com.fileitup.fisixengine.core.EngineObject;
	import com.fileitup.fisixengine.primitives.Particle;
	import com.fileitup.fisixengine.graphics.DisplayAttacher2;
	import flash.events.KeyboardEvent;
	import flash.events.Event;
	import flash.ui.Keyboard;
	import fisixObjects.TruckV2;
	import com.fileitup.fisixengine.particles.WheelParticle;
	import com.fileitup.fisixengine.graphics.DisplayAttacher;

	[SWF(width='450',height='300',backgroundColor='0xFFFFFF',frameRate='31')]
	public class MonsterTruck extends Sprite
	{
		
		private var _engine:FisixEngine;
		
		private var _game:Sprite;
		private var _renderer:Sprite;
		
		private var _truck:TruckV2;
		
		//keys
		private var _forward:Boolean = false;
		private var _backward:Boolean = false;
		
		//graphics
		[Embed(source="../assets/rightWheel.png")]
		public var rightWheel:Class;
		[Embed(source="../assets/leftWheel.png")]
		public var leftWheel:Class;
		[Embed(source="../assets/carBody.png")]
		public var carBody:Class;
		[Embed(source="../assets/track.png")]
		public var carTrack:Class;
		[Embed(source="../assets/pebble.png")]
		public var pebble:Class;
		
		public function MonsterTruck()
		{
			//sprites
			_game = new Sprite();
			_renderer = new Sprite();
			addChildAt(_game, 0);
			addChildAt(_renderer, 1);
			//setup
			setupEngine();
			drawBack();
			addGraphics();
			addProps();
			startEngine();
			addControls();
			
		}
		
		private function setupEngine():void
		{
			//engine
			_engine = new FisixEngine();
			_engine.setReactionMode(ReactionModes.PHYSICAL);
			_engine.setGravity(0, 31);
			_engine.setFriction(1, true);
			//starting surface
			drawSurfaces();
			//add truck
			_truck = new TruckV2(200, 100);
			_engine.addObject(_truck);
		}
		
		private function startEngine():void
		{
			//_engine.setRender(true);
			//_engine.setRenderGraphics(_renderer.graphics);
			_engine.startEngine(stage.frameRate);
		}
		
		private function addControls():void 
		{
			stage.addEventListener(KeyboardEvent.KEY_DOWN, trapKeys);
			stage.addEventListener(KeyboardEvent.KEY_UP, trapKeysUp);
			stage.addEventListener(Event.ENTER_FRAME, onFrame);
		}
		
		private function trapKeys(event:KeyboardEvent):void {
			if (event.keyCode == 39){
				_forward = true;
				_backward = false;
			} else if (event.keyCode == 37){
				_forward = false;
				_backward = true;
			}
		}
		
		private function trapKeysUp(event:KeyboardEvent):void {
			if (event.keyCode == 39){
				_forward = false;
			} else if (event.keyCode == 37){
				_backward = false;
			}
		}
		
		private function onFrame(event:Event):void
		{
			//controls
			if (_forward){
				if (_truck.leftWheel.velocity.x < 17){
					_truck.leftWheel.addVelocity(new Vector(1, 0));
					_truck.rightWheel.addVelocity(new Vector(1, 0));
				}
			} else if (_backward){
				if (_truck.leftWheel.velocity.x > -12){
					_truck.leftWheel.addVelocity(new Vector(-1, 0));
					_truck.rightWheel.addVelocity(new Vector(-1, 0));
				}
			}
			//scroll
			var xpos:Number = ((_game.getChildByName('truck').x - 150) * -1 );
			if (_game.x < xpos && Math.abs(_game.x - xpos) > 1){
				var pos:Number = (xpos - _game.x)/8;
				if (_game.x < -10){
				 	_game.x += pos;
					_renderer.x += pos;
				}
			} else if (_game.x > xpos && Math.abs(_game.x - xpos) > 1){
				var posm:Number = (_game.x - xpos)/8;
				if (_game.x > -3750 + 450){
					_game.x -= posm;
					_renderer.x -= posm;
				}
			}
		}
		
		private function addGraphics():void 
		{
			//add graphics for body
			var bmpc:Bitmap = new carBody() as Bitmap;
			var sprc:Sprite = new Sprite();
			sprc.name = 'truck';
			bmpc.x = -8;
			bmpc.y = -18;
			sprc.addChild(bmpc);
			_game.addChild(sprc);
			//add graphics for wheels
			var bmpr:Bitmap = new rightWheel() as Bitmap;
			var bmpl:Bitmap = new leftWheel() as Bitmap;
			bmpr.x = bmpl.x = - 21;
			bmpr.y = bmpl.y = - 21;
			var sprr:Sprite = new Sprite();
			var sprl:Sprite = new Sprite();
			sprr.addChild(bmpr);
			sprl.addChild(bmpl);
			_game.addChild(sprr);
			_game.addChild(sprl);
			//attach graphics
			var wr_bind:WheelAttacher = _engine.newWheelAttacher(sprr, _truck.rightWheel);
			var wl_bind:WheelAttacher = _engine.newWheelAttacher(sprl, _truck.leftWheel);
			var cr_bind:DisplayAttacher2 = _engine.newDisplayAttacher2(sprc, _truck.bum.pos, _truck.nose.pos);
		}
		
		private function addProps():void 
		{
			for (var n:Number = 0; n < 5; n++){
				var rock:WheelParticle = _engine.newWheelParticle(1760+((20+Math.random()+40)*n), 200, 9);
				var pb:Bitmap = new pebble() as Bitmap;
				var pbh:Sprite = new Sprite();
				pbh.addChild(pb);
				pb.x = -9;
				pb.y = -9;
				_game.addChild(pbh);
				_engine.newWheelAttacher(pbh, rock);
			}
		}
		
		private function drawBack():void
		{
			var background:Sprite = new Sprite();
			var matrix:Matrix = new Matrix();
			matrix.createGradientBox(3770, 300, Math.PI/2);
			background.graphics.beginGradientFill(GradientType.LINEAR, [0x003366, 0x0099FF], [1,1], [0, 255], matrix);
			background.graphics.drawRect(0, 0, 3770, 300);
			_game.addChildAt(background, 0);
			//track
			var track:Bitmap = new carTrack() as Bitmap;
			track.y = 135;
			track.x = -12;
			_game.addChildAt(track, 1);
		}
		
		private function drawSurfaces():void 
		{
			var surface00:Surface = _engine.newSurface(new Vector(0, 0), new Vector(0, 300), 30);
			var surface0:Surface = _engine.newSurface(new Vector(0, 300), new Vector(408, 302), 20);
			var surface1:Surface = _engine.newSurface(new Vector(408, 302), new Vector(451, 289), 20);
			var surface2:Surface = _engine.newSurface(new Vector(451, 289), new Vector(484, 279), 20);
			var surface3:Surface = _engine.newSurface(new Vector(484, 279), new Vector(489, 279), 20);
			var surface4:Surface = _engine.newSurface(new Vector(489, 279), new Vector(517, 282), 20);
			var surface5:Surface = _engine.newSurface(new Vector(517, 282), new Vector(568, 280), 20);
			var surface6:Surface = _engine.newSurface(new Vector(568, 280), new Vector(585, 283), 20);
			var surface7:Surface = _engine.newSurface(new Vector(585, 283), new Vector(610, 287), 20);
			var surface8:Surface = _engine.newSurface(new Vector(610, 287), new Vector(626, 296), 20);
			var surface9:Surface = _engine.newSurface(new Vector(626, 296), new Vector(641, 302), 20);
			var surface10:Surface = _engine.newSurface(new Vector(641, 302), new Vector(654, 299), 20);
			var surface11:Surface = _engine.newSurface(new Vector(654, 299), new Vector(663, 290), 20);
			var surface12:Surface = _engine.newSurface(new Vector(663, 290), new Vector(670, 289), 20);
			var surface13:Surface = _engine.newSurface(new Vector(670, 289), new Vector(687, 300), 20);
			var surface14:Surface = _engine.newSurface(new Vector(687, 300), new Vector(722, 305), 20);
			var surface15:Surface = _engine.newSurface(new Vector(722, 305), new Vector(742, 289), 20);
			var surface16:Surface = _engine.newSurface(new Vector(742, 289), new Vector(761, 284), 20);
			var surface17:Surface = _engine.newSurface(new Vector(761, 284), new Vector(763, 283), 20);
			var surface18:Surface = _engine.newSurface(new Vector(763, 283), new Vector(781, 287), 20);
			var surface19:Surface = _engine.newSurface(new Vector(781, 287), new Vector(812, 304), 20);
			var surface20:Surface = _engine.newSurface(new Vector(812, 304), new Vector(1064, 304), 20);
			var surface21:Surface = _engine.newSurface(new Vector(1064, 304), new Vector(1082, 289), 20);
			var surface22:Surface = _engine.newSurface(new Vector(1082, 289), new Vector(1094, 284), 20);
			var surface23:Surface = _engine.newSurface(new Vector(1094, 284), new Vector(1111, 291), 20);
			var surface24:Surface = _engine.newSurface(new Vector(1111, 291), new Vector(1124, 295), 20);
			var surface25:Surface = _engine.newSurface(new Vector(1124, 295), new Vector(1147, 274), 20);
			var surface26:Surface = _engine.newSurface(new Vector(1147, 274), new Vector(1157, 268), 20);
			var surface27:Surface = _engine.newSurface(new Vector(1157, 268), new Vector(1171, 275), 20);
			var surface28:Surface = _engine.newSurface(new Vector(1171, 275), new Vector(1199, 292), 20);
			var surface29:Surface = _engine.newSurface(new Vector(1199, 292), new Vector(1212, 294), 20);
			var surface30:Surface = _engine.newSurface(new Vector(1212, 294), new Vector(1232, 285), 20);
			var surface31:Surface = _engine.newSurface(new Vector(1232, 285), new Vector(1249, 277), 20);
			var surface32:Surface = _engine.newSurface(new Vector(1249, 277), new Vector(1276, 281), 20);
			var surface33:Surface = _engine.newSurface(new Vector(1276, 281), new Vector(1299, 289), 20);
			var surface34:Surface = _engine.newSurface(new Vector(1299, 289), new Vector(1331, 305), 20);
			var surface35:Surface = _engine.newSurface(new Vector(1331, 305), new Vector(1453, 303), 20);
			var surface36:Surface = _engine.newSurface(new Vector(1453, 303), new Vector(1496, 285), 20);
			var surface37:Surface = _engine.newSurface(new Vector(1496, 285), new Vector(1552, 265), 20);
			var surface38:Surface = _engine.newSurface(new Vector(1552, 265), new Vector(1641, 245), 20);
			var surface39:Surface = _engine.newSurface(new Vector(1641, 245), new Vector(1716, 233), 20);
			var surface40:Surface = _engine.newSurface(new Vector(1716, 233), new Vector(1758, 234), 20);
			var surface41:Surface = _engine.newSurface(new Vector(1758, 234), new Vector(1776, 271), 20);
			var surface42:Surface = _engine.newSurface(new Vector(1776, 271), new Vector(1814, 285), 20);
			var surface43:Surface = _engine.newSurface(new Vector(1814, 285), new Vector(1835, 274), 20);
			var surface44:Surface = _engine.newSurface(new Vector(1835, 274), new Vector(1869, 269), 20);
			var surface45:Surface = _engine.newSurface(new Vector(1869, 269), new Vector(1888, 253), 20);
			var surface46:Surface = _engine.newSurface(new Vector(1888, 253), new Vector(1906, 242), 20);
			var surface47:Surface = _engine.newSurface(new Vector(1906, 242), new Vector(1934, 250), 20);
			var surface48:Surface = _engine.newSurface(new Vector(1934, 250), new Vector(1980, 260), 20);
			var surface49:Surface = _engine.newSurface(new Vector(1980, 260), new Vector(2068, 304), 20);
			var surface50:Surface = _engine.newSurface(new Vector(2068, 304), new Vector(2290, 302), 20);
			var surface51:Surface = _engine.newSurface(new Vector(2290, 302), new Vector(2325, 263), 20);
			var surface52:Surface = _engine.newSurface(new Vector(2325, 263), new Vector(2361, 261), 20);
			var surface53:Surface = _engine.newSurface(new Vector(2361, 261), new Vector(2403, 282), 20);
			var surface54:Surface = _engine.newSurface(new Vector(2403, 282), new Vector(2439, 261), 20);
			var surface55:Surface = _engine.newSurface(new Vector(2439, 261), new Vector(2466, 236), 20);
			var surface56:Surface = _engine.newSurface(new Vector(2466, 236), new Vector(2491, 224), 20);
			var surface57:Surface = _engine.newSurface(new Vector(2491, 224), new Vector(2524, 241), 20);
			var surface58:Surface = _engine.newSurface(new Vector(2524, 241), new Vector(2554, 256), 20);
			var surface59:Surface = _engine.newSurface(new Vector(2554, 256), new Vector(2586, 274), 20);
			var surface60:Surface = _engine.newSurface(new Vector(2586, 274), new Vector(2598, 302), 20);
			var surface61:Surface = _engine.newSurface(new Vector(2598, 302), new Vector(2620, 305), 20);
			var surface62:Surface = _engine.newSurface(new Vector(2620, 305), new Vector(2624, 299), 20);
			var surface63:Surface = _engine.newSurface(new Vector(2624, 299), new Vector(2625, 291), 20);
			var surface64:Surface = _engine.newSurface(new Vector(2625, 291), new Vector(2627, 286), 20);
			var surface65:Surface = _engine.newSurface(new Vector(2627, 286), new Vector(2647, 283), 20);
			var surface66:Surface = _engine.newSurface(new Vector(2647, 283), new Vector(2667, 274), 20);
			var surface67:Surface = _engine.newSurface(new Vector(2667, 274), new Vector(2669, 273), 20);
			var surface68:Surface = _engine.newSurface(new Vector(2669, 273), new Vector(2674, 271), 20);
			var surface69:Surface = _engine.newSurface(new Vector(2674, 271), new Vector(2702, 269), 20);
			var surface70:Surface = _engine.newSurface(new Vector(2702, 269), new Vector(2737, 287), 20);
			var surface71:Surface = _engine.newSurface(new Vector(2737, 287), new Vector(2757, 299), 20);
			var surface72:Surface = _engine.newSurface(new Vector(2757, 299), new Vector(2796, 292), 20);
			var surface73:Surface = _engine.newSurface(new Vector(2796, 292), new Vector(2832, 306), 20);
			var surface74:Surface = _engine.newSurface(new Vector(2832, 306), new Vector(2835, 305), 20);
			var surface75:Surface = _engine.newSurface(new Vector(2835, 305), new Vector(2869, 301), 20);
			var surface76:Surface = _engine.newSurface(new Vector(2869, 301), new Vector(3095, 300), 20);
			var surface77:Surface = _engine.newSurface(new Vector(3095, 300), new Vector(3137, 286), 20);
			var surface78:Surface = _engine.newSurface(new Vector(3137, 286), new Vector(3149, 275), 20);
			var surface79:Surface = _engine.newSurface(new Vector(3149, 275), new Vector(3168, 251), 20);
			var surface80:Surface = _engine.newSurface(new Vector(3168, 251), new Vector(3196, 226), 20);
			var surface81:Surface = _engine.newSurface(new Vector(3196, 226), new Vector(3207, 226), 20);
			var surface82:Surface = _engine.newSurface(new Vector(3207, 226), new Vector(3233, 235), 20);
			var surface83:Surface = _engine.newSurface(new Vector(3233, 235), new Vector(3242, 232), 20);
			var surface84:Surface = _engine.newSurface(new Vector(3242, 232), new Vector(3241, 222), 20);
			var surface85:Surface = _engine.newSurface(new Vector(3241, 222), new Vector(3245, 208), 20);
			var surface86:Surface = _engine.newSurface(new Vector(3245, 208), new Vector(3250, 199), 20);
			var surface87:Surface = _engine.newSurface(new Vector(3250, 199), new Vector(3305, 183), 20);
			var surface88:Surface = _engine.newSurface(new Vector(3305, 183), new Vector(3340, 189), 20);
			var surface89:Surface = _engine.newSurface(new Vector(3340, 189), new Vector(3391, 228), 20);
			var surface90:Surface = _engine.newSurface(new Vector(3391, 228), new Vector(3448, 288), 20);
			var surface91:Surface = _engine.newSurface(new Vector(3448, 288), new Vector(3464, 297), 20);
			var surface92:Surface = _engine.newSurface(new Vector(3464, 297), new Vector(3515, 302), 20);
			var surface93:Surface = _engine.newSurface(new Vector(3515, 302), new Vector(3748, 303), 20);
			var surface94:Surface = _engine.newSurface(new Vector(3748, 303), new Vector(3748, 0), 30);
		}
	}
}
